var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { html } from "lit";
import { property, query, state } from "lit/decorators.js";
import { classMap } from "lit/directives/class-map.js";
import { repeat } from "lit/directives/repeat.js";
import { computeFlipOffsetPosition, deepEqual, FtLitElement } from "@fluid-topics/ft-wc-utils";
import { FtTypography, FtTypographyBody2, FtTypographyCaption } from "@fluid-topics/ft-typography";
import { FtInputLabel } from "@fluid-topics/ft-input-label";
import { FtRipple } from "@fluid-topics/ft-ripple";
import { FtIcon } from "@fluid-topics/ft-icon";
import { styles } from "./ft-select.styles";
class FtSelectOption extends FtLitElement {
    constructor() {
        super(...arguments);
        this.label = "";
        this.value = null;
        this.selected = false;
    }
    render() {
        return html ``;
    }
    update(changedProperties) {
        super.update(changedProperties);
        this.dispatchEvent(new CustomEvent("option-change", { detail: this, bubbles: true }));
    }
}
FtSelectOption.elementDefinitions = {};
__decorate([
    property({ type: String })
], FtSelectOption.prototype, "label", void 0);
__decorate([
    property({ type: Object, converter: value => value })
], FtSelectOption.prototype, "value", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], FtSelectOption.prototype, "selected", void 0);
export { FtSelectOption };
class FtSelect extends FtLitElement {
    constructor() {
        super(...arguments);
        this.label = "";
        this.ariaLabel = "";
        this.helper = "";
        this.outlined = false;
        this.disabled = false;
        this.error = false;
        this.fixedMenuPosition = false;
        this.options = [];
        this.optionsDisplayed = false;
        this.focusOptions = false;
        this.hideOptions = (e) => this.optionsDisplayed = this.optionsDisplayed && e.composedPath().includes(this.container);
    }
    render() {
        var _a, _b, _c, _d, _e;
        let optionsDisplayed = this.hasOptionsMenuOpen;
        let disabled = this.disabled || !this.hasOptions;
        const hasOptionSelected = ((_a = this.selectedOption) === null || _a === void 0 ? void 0 : _a.value) != null || ((_c = (_b = this.selectedOption) === null || _b === void 0 ? void 0 : _b.label) !== null && _c !== void 0 ? _c : "").length > 0;
        const classes = {
            "ft-select": true,
            "ft-select--filled": !this.outlined,
            "ft-select--outlined": this.outlined,
            "ft-select--disabled": disabled,
            "ft-select--options-displayed": optionsDisplayed,
            "ft-select--has-option-selected": hasOptionSelected,
            "ft-select--no-label": !this.label,
            "ft-select--fixed": this.fixedMenuPosition,
            "ft-select--in-error": this.error
        };
        return html `
            <div class="${classMap(classes)}" part="container">
                <div class="ft-select--main-panel" part="main-panel">
                    <ft-input-label text="${this.label}"
                                    part="label"
                                    ?disabled=${disabled}
                                    ?outlined=${this.outlined}
                                    ?raised=${hasOptionSelected || optionsDisplayed}
                                    ?error=${this.error}
                    ></ft-input-label>
                    <div class="ft-select--input-panel"
                         part="selected-value"
                         tabindex="${disabled ? "-1" : "0"}"
                         @click=${() => {
            this.optionsDisplayed = !this.optionsDisplayed;
            this.focusOptions = this.optionsDisplayed;
        }}
                         @keydown=${this.onMainPanelKeyDown}
                         aria-label="${this.ariaLabel}"
                         aria-haspopup="listbox"
                         aria-expanded="${optionsDisplayed}"
                         role="combobox">
                        <ft-ripple ?disabled=${disabled} ?activated=${!this.outlined}></ft-ripple>
                        <ft-typography variant="body1" class="ft-select--selected-option">
                            ${(_e = (_d = this.selectedOption) === null || _d === void 0 ? void 0 : _d.label) !== null && _e !== void 0 ? _e : ""}
                        </ft-typography>
                        <ft-icon>${optionsDisplayed ? "thin_arrow_up" : "thin_arrow"}</ft-icon>
                    </div>
                    <div class="ft-select--options"
                         part="options"
                         @keydown=${this.onOptionsKeyDown}
                         innerrole="listbox">
                        ${repeat(this.options, option => option.value, option => this.renderOption(option))}
                    </div>
                </div>
                ${this.helper ? html `
                    <ft-typography class="ft-select--helper-text" variant="caption">${this.helper}
                    </ft-typography>` : undefined}
            </div>
            <slot @slotchange=${this.updateOptionsFromSlot}
                  @option-change=${this.updateOptionsFromSlot}
            ></slot>
        `;
    }
    renderOption(option) {
        let selected = this.selectedOption === option;
        const classes = {
            "ft-select--option": true,
            "ft-select--option-selected": selected,
            "ft-typography--body2": true
        };
        return html `
            <div class="${classMap(classes)}"
                 part="option"
                 tabindex="0"
                 data-value="${option.value}"
                 @keydown=${(e) => this.onOptionKeyDown(e, option)}
                 @click=${() => this.selectOption(option)}>
                <ft-ripple ?primary=${selected} ?activated=${selected}></ft-ripple>
                <span>${option.label}</span>
            </div>
        `;
    }
    update(props) {
        super.update(props);
        if (props.has("options")) {
            this.selectedOption = this.options.filter(o => o.selected)[0];
        }
        if (props.has("selectedOption")) {
            this.optionsDisplayed = false;
        }
    }
    contentAvailableCallback(props) {
        var _a, _b;
        super.contentAvailableCallback(props);
        if (props.has("focusOptions") && this.focusOptions) {
            (_b = ((_a = this.selectedOptionElement) !== null && _a !== void 0 ? _a : this.firstOption)) === null || _b === void 0 ? void 0 : _b.focus();
            this.focusOptions = false;
        }
        if (props.has("optionsDisplayed") && this.hasOptionsMenuOpen) {
            this.optionsMenu.style.width = this.mainPanel.getBoundingClientRect().width + "px";
            const fallbackPlacements = ["bottom", "top"];
            computeFlipOffsetPosition(this.mainPanel, this.optionsMenu, "bottom", fallbackPlacements, "fixed")
                .then(({ x, y }) => {
                this.optionsMenu.style.left = `${x}px`;
                this.optionsMenu.style.top = `${y}px`;
            });
        }
    }
    get hasOptionsMenuOpen() {
        return !this.disabled && this.optionsDisplayed && this.hasOptions;
    }
    get hasOptions() {
        return this.options.length > 0;
    }
    updateOptionsFromSlot(e) {
        var _a;
        e.stopPropagation();
        let optionsFromSlot = (_a = this.optionsSlot) === null || _a === void 0 ? void 0 : _a.assignedElements().map(n => n);
        if (optionsFromSlot && optionsFromSlot.length > 0) {
            this.options = optionsFromSlot;
        }
    }
    onMainPanelKeyDown(e) {
        switch (e.key) {
            // @ts-ignore
            case " ":
                e.preventDefault();
                e.stopPropagation();
            // No break
            case "Enter":
                this.optionsDisplayed = !this.optionsDisplayed;
                this.focusOptions = true;
                break;
            case "ArrowUp":
            case "ArrowDown":
                e.preventDefault();
                e.stopPropagation();
                this.optionsDisplayed = true;
                this.focusOptions = true;
                break;
        }
    }
    onOptionsKeyDown(e) {
        var _a, _b, _c, _d, _e;
        let optionToFocus;
        switch (e.key) {
            case "Escape":
                this.optionsDisplayed = false;
                (_a = this.mainPanel) === null || _a === void 0 ? void 0 : _a.focus();
                break;
            case "Tab":
                this.optionsDisplayed = false;
                break;
            case "ArrowUp":
                e.preventDefault();
                e.stopPropagation();
                optionToFocus = (_c = (_b = this.focusedOption) === null || _b === void 0 ? void 0 : _b.previousElementSibling) !== null && _c !== void 0 ? _c : this.lastOption;
                break;
            case "ArrowDown":
                e.preventDefault();
                e.stopPropagation();
                optionToFocus = (_e = (_d = this.focusedOption) === null || _d === void 0 ? void 0 : _d.nextElementSibling) !== null && _e !== void 0 ? _e : this.firstOption;
                break;
        }
        optionToFocus === null || optionToFocus === void 0 ? void 0 : optionToFocus.focus();
    }
    onOptionKeyDown(e, option) {
        var _a;
        if (e.key === "Enter" || e.key === " ") {
            e.preventDefault();
            e.stopPropagation();
            this.selectOption(option);
            this.optionsDisplayed = false;
            (_a = this.mainPanel) === null || _a === void 0 ? void 0 : _a.focus();
        }
    }
    selectOption(option) {
        var _a, _b;
        if (!deepEqual((_a = this.selectedOption) === null || _a === void 0 ? void 0 : _a.value, option.value)) {
            this.selectedOption = option;
            for (let otherOption of this.options) {
                otherOption.selected = otherOption === option;
            }
            this.dispatchEvent(new CustomEvent("change", { detail: (_b = this.selectedOption) === null || _b === void 0 ? void 0 : _b.value }));
        }
        this.optionsDisplayed = false;
    }
    connectedCallback() {
        super.connectedCallback();
        document.addEventListener("click", this.hideOptions);
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        document.removeEventListener("click", this.hideOptions);
    }
}
FtSelect.elementDefinitions = {
    "ft-input-label": FtInputLabel,
    "ft-typography": FtTypography,
    "ft-ripple": FtRipple,
    "ft-icon": FtIcon,
};
FtSelect.styles = [
    FtTypographyBody2,
    FtTypographyCaption,
    styles
];
__decorate([
    property({ type: String })
], FtSelect.prototype, "label", void 0);
__decorate([
    property({ type: String })
], FtSelect.prototype, "ariaLabel", void 0);
__decorate([
    property({ type: String })
], FtSelect.prototype, "helper", void 0);
__decorate([
    property({ type: Boolean })
], FtSelect.prototype, "outlined", void 0);
__decorate([
    property({ type: Boolean })
], FtSelect.prototype, "disabled", void 0);
__decorate([
    property({ type: Boolean })
], FtSelect.prototype, "error", void 0);
__decorate([
    property({ type: Boolean })
], FtSelect.prototype, "fixedMenuPosition", void 0);
__decorate([
    property({ type: Array })
], FtSelect.prototype, "options", void 0);
__decorate([
    state()
], FtSelect.prototype, "selectedOption", void 0);
__decorate([
    state()
], FtSelect.prototype, "optionsDisplayed", void 0);
__decorate([
    state()
], FtSelect.prototype, "focusOptions", void 0);
__decorate([
    query(".ft-select")
], FtSelect.prototype, "container", void 0);
__decorate([
    query(".ft-select--options")
], FtSelect.prototype, "optionsMenu", void 0);
__decorate([
    query(".ft-select--input-panel")
], FtSelect.prototype, "mainPanel", void 0);
__decorate([
    query(".ft-select--option:first-child")
], FtSelect.prototype, "firstOption", void 0);
__decorate([
    query(".ft-select--option:focus")
], FtSelect.prototype, "focusedOption", void 0);
__decorate([
    query(".ft-select--option.ft-select--option-selected")
], FtSelect.prototype, "selectedOptionElement", void 0);
__decorate([
    query(".ft-select--option:last-child")
], FtSelect.prototype, "lastOption", void 0);
__decorate([
    query("slot")
], FtSelect.prototype, "optionsSlot", void 0);
export { FtSelect };
