import { css } from "lit";
import { designSystemVariables, FtCssVariableFactory, setVariable } from "@fluid-topics/ft-wc-utils";
import { FtInputLabelCssVariables } from "@fluid-topics/ft-input-label/build/ft-input-label.styles";
import { FtTypographyBody1CssVariables } from "@fluid-topics/ft-typography/build/ft-typography.styles";
import { FtRippleCssVariables } from "@fluid-topics/ft-ripple/build/ft-ripple.styles";
import { FtIconCssVariables } from "@fluid-topics/ft-icon/build/ft-icon.styles";
export const FtSelectCssVariables = {
    labelSize: FtCssVariableFactory.create("--ft-select-label-size", "", "SIZE", "11px"),
    selectedOptionSize: FtCssVariableFactory.create("--ft-select-selected-option-size", "", "SIZE", "14px"),
    verticalSpacing: FtCssVariableFactory.create("--ft-select-vertical-spacing", "", "SIZE", "4px"),
    horizontalSpacing: FtCssVariableFactory.create("--ft-select-horizontal-spacing", "", "SIZE", "16px"),
    optionsHeight: FtCssVariableFactory.create("--ft-select-options-height", "", "SIZE", "70vh"),
    selectedOptionColor: FtCssVariableFactory.extend("--ft-select-selected-option-color", "", designSystemVariables.colorOnSurface),
    helperColor: FtCssVariableFactory.extend("--ft-select-helper-color", "", designSystemVariables.colorOnSurfaceMedium),
    optionsColor: FtCssVariableFactory.extend("--ft-select-options-color", "", designSystemVariables.colorOnSurface),
    optionsZIndex: FtCssVariableFactory.create("--ft-select-options-z-index", "", "NUMBER", "3"),
    backgroundColor: FtCssVariableFactory.extend("--ft-select-background-color", "", designSystemVariables.colorSurface),
    colorOnSurfaceDisabled: FtCssVariableFactory.external(designSystemVariables.colorOnSurfaceDisabled, "Design system"),
    colorPrimary: FtCssVariableFactory.external(designSystemVariables.colorPrimary, "Design system"),
    borderRadiusS: FtCssVariableFactory.external(designSystemVariables.borderRadiusS, "Design system"),
    elevation02: FtCssVariableFactory.external(designSystemVariables.elevation02, "Design system"),
    colorError: FtCssVariableFactory.external(designSystemVariables.colorError, "Design system"),
};
//language=css
export const styles = css `
    *:focus {
        outline: none;
    }

    .ft-select {
        display: flex;
        flex-direction: column;
        align-items: stretch;
    }

    .ft-select--main-panel {
        position: relative;
        display: flex;
        height: calc(4 * ${FtSelectCssVariables.verticalSpacing} + ${FtSelectCssVariables.labelSize} + ${FtSelectCssVariables.selectedOptionSize});
    }

    .ft-select--input-panel {
        flex-grow: 1;
        position: relative;
        display: flex;
        align-items: center;
        overflow: hidden;
        padding-left: ${FtSelectCssVariables.horizontalSpacing};
        padding-right: ${FtSelectCssVariables.horizontalSpacing};
        gap: 8px;
        color: ${FtSelectCssVariables.selectedOptionColor};
        ${setVariable(FtRippleCssVariables.color, FtSelectCssVariables.selectedOptionColor)};
    }

    .ft-select--input-panel,
    .ft-select--option {
        ${setVariable(FtRippleCssVariables.opacityContentOnSurfaceHover, "0.08")};
        ${setVariable(FtRippleCssVariables.opacityContentOnSurfacePressed, "0.04")};
    }

    .ft-select--input-panel > *:not(ft-ripple),
    .ft-select--option > *:not(ft-ripple) {
        position: relative;
    }

    .ft-select--disabled .ft-select--input-panel,
    .ft-select--disabled .ft-select--option {
        color: ${FtSelectCssVariables.colorOnSurfaceDisabled};
    }

    .ft-select:not(.ft-select--disabled) .ft-select--input-panel,
    .ft-select:not(.ft-select--disabled) .ft-select--option {
        cursor: pointer;
    }

    ft-input-label {
        ${setVariable(FtInputLabelCssVariables.fontSize, FtSelectCssVariables.selectedOptionSize)};
        ${setVariable(FtInputLabelCssVariables.raisedFontSize, FtSelectCssVariables.labelSize)};
        ${setVariable(FtInputLabelCssVariables.verticalSpacing, FtSelectCssVariables.verticalSpacing)};
        ${setVariable(FtInputLabelCssVariables.horizontalSpacing, FtSelectCssVariables.horizontalSpacing)};
        ${setVariable(FtInputLabelCssVariables.backgroundColor, FtSelectCssVariables.backgroundColor)};
        ${setVariable(FtInputLabelCssVariables.labelMaxWidth, `calc(100% - ${FtIconCssVariables.size} - 8px)`)};
    }

    .ft-select:not(.ft-select--disabled):focus-within ft-icon {
        color: ${FtSelectCssVariables.colorPrimary};
    }

    .ft-select:not(.ft-select--disabled):focus-within ft-input-label {
        ${setVariable(FtInputLabelCssVariables.borderColor, FtSelectCssVariables.colorPrimary)};
        ${setVariable(FtInputLabelCssVariables.textColor, FtSelectCssVariables.colorPrimary)};
    }

    .ft-select--selected-option {
        display: block;
        flex-grow: 1;
        flex-shrink: 1;
        overflow: hidden;
        white-space: nowrap;
        text-overflow: ellipsis;
        ${setVariable(FtTypographyBody1CssVariables.fontSize, FtSelectCssVariables.selectedOptionSize)};
        ${setVariable(FtTypographyBody1CssVariables.lineHeight, FtSelectCssVariables.selectedOptionSize)};
    }

    .ft-select--in-error .ft-select--selected-option {
        color: ${FtSelectCssVariables.colorError};
    }

    ft-icon {
        flex-shrink: 0;
    }

    slot {
        display: none;
    }

    .ft-select--filled .ft-select--input-panel {
        border-radius: ${FtSelectCssVariables.borderRadiusS} ${FtSelectCssVariables.borderRadiusS} 0 0;
    }

    .ft-select--filled:not(.ft-select--no-label) .ft-select--selected-option {
        align-self: stretch;
        padding-top: calc(${FtSelectCssVariables.labelSize} + 2 * ${FtSelectCssVariables.verticalSpacing});
    }

    .ft-select--outlined .ft-select--input-panel {
        border-radius: ${FtSelectCssVariables.borderRadiusS};
    }

    .ft-select--options {
        display: none;
        position: absolute;
        top: 100%;
        color: ${FtSelectCssVariables.optionsColor};
        background: ${FtSelectCssVariables.backgroundColor};
        z-index: ${FtSelectCssVariables.optionsZIndex};
        box-shadow: ${FtSelectCssVariables.elevation02};
    }

    .ft-select--fixed .ft-select--options {
        position: fixed;
    }

    .ft-select--options-displayed .ft-select--options {
        display: block;
        max-height: ${FtSelectCssVariables.optionsHeight};
        overflow-y: auto;
    }

    .ft-select--option {
        position: relative;
        padding: 4px ${FtSelectCssVariables.horizontalSpacing};
        min-height: 32px;
        display: flex;
        align-items: center;
    }

    .ft-select--option span {
        text-overflow: ellipsis;
        overflow: hidden;
    }

    .ft-select--helper-text {
        padding: 0 ${FtSelectCssVariables.horizontalSpacing};
        color: ${FtSelectCssVariables.helperColor};
    }

    .ft-select--in-error .ft-select--helper-text {
        color: ${FtSelectCssVariables.colorError};
    }
`;
